function menuSelectionBuffer_Callback(obj, parameter)
% function menuSelectionBuffer_Callback(obj, parameter)
% a callback to Menu->Selection to Buffer... to Copy/Paste/Clear of the selection of the shown layer
%
% Parameters:
% parameter: a string that defines image source:
% - ''copy'', store the selection from the current layer to a buffer
% - ''paste'', paste the selection from the buffer to the current layer
% - ''clear'', clear selection buffer

% Copyright (C) 20.11.2016, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 


% do nothing is selection is disabled
if obj.mibModel.preferences.disableSelection == 1; return; end;

switch parameter
    case 'copy'
        obj.mibModel.I{obj.mibModel.Id}.storedSelection = cell2mat(obj.mibModel.getData2D('selection'));
    case 'paste'
        if isempty(obj.mibModel.I{obj.mibModel.Id}.storedSelection)
            msgbox(sprintf('Error!\nThe buffer is empty!'),'Error!','error','modal');
            return;
        end;
        currSelection = cell2mat(obj.mibModel.getData2D('selection'));
        if min(size(currSelection) == size(obj.mibModel.I{obj.mibModel.Id}.storedSelection)) == 1
            obj.mibModel.mibDoBackup('selection', 0);
            obj.mibModel.setData2D('selection', bitor(obj.mibModel.I{obj.mibModel.Id}.storedSelection, currSelection));
            obj.plotImage(0);
        else
            msgbox(sprintf('Error!\nThe size of the buffered and current selections mismatch!\nTry to change the orientation of the dataset...'),'Error!','error','modal');
        end
    case 'clear'
        obj.mibModel.I{obj.mibModel.Id}.storedSelection = [];
end
end