function convertModel(obj, type)
% function convertModel(obj, type)
% Convert model from obj.modelType==63 to obj.modelType==255 and other way around
%
% @note The current type is defined with obj.modelType
%
% Parameters:
% type: [optional] a double with type of a new model: 63 (63 materials max) or 255 (255 materials max)
%
% Return values:
%

%| 
% @b Examples:
% @code obj.mibModel.I{obj.mibModel.Id}.convertModel(255);  // convert model to the obj.modelType==255 type @endcode

% Copyright (C) 12.01.2017 Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% part of Microscopy Image Browser, http:\\mib.helsinki.fi
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.

% Updates
% 

if nargin < 2
    if obj.modelType == 255
        type = 63;
    else
        type = 255;
    end
end
if type == 255 && obj.modelType == 255; return; end;
if type == 63 && obj.modelType == 63; return; end;

wb = waitbar(0, sprintf('Converting the model to the "%d" type\n\nPlease wait...', type), 'Name', 'Converting the model', 'WindowStyle', 'modal');
if type == 63     % convert from 255 to 63
    if ~isnan(obj.selection{1}(1))
        if isnan(obj.model{1}(1)); obj.model{1} = zeros([size(obj.img{1},1) size(obj.img{1},2) size(obj.img{1},4) size(obj.img{1},5)], 'uint8'); end; % create new model
        obj.model{1}(obj.selection{1}==1) = bitset(obj.model{1}(obj.selection{1}==1), 8, 1);    % generate selection layer
        waitbar(0.4, wb);
        if obj.maskExist == 1
            obj.model{1}(obj.maskImg{1}==1) = bitset(obj.model{1}(obj.maskImg{1}==1), 7, 1);    % generate mask layer
        end
        waitbar(0.8, wb);
        obj.maskExist = 1;
        obj.maskImg{1} = NaN;
        obj.selection{1} = NaN;
    end
    obj.modelType = 63;
    waitbar(1, wb);
else                        % convert from 63 to 255
    waitbar(0.3, wb);
    if ~isnan(obj.model{1}(1))     % convert when the layers are present
        obj.selection{1} = zeros([size(obj.img{1},1) size(obj.img{1},2) size(obj.img{1},4) size(obj.img{1},5)], 'uint8');
        obj.maskImg{1} = zeros([size(obj.img{1},1) size(obj.img{1},2) size(obj.img{1},4) size(obj.img{1},5)], 'uint8');
        obj.selection{1} = bitand(obj.model{1}, 128)/128;     % generate selection
        obj.maskImg{1} = bitand(obj.model{1}, 64)/64;     % generate mask
        waitbar(0.6, wb);
        obj.model{1} = bitand(obj.model{1}, 63);  % clear mask and selection from the model
        waitbar(0.9, wb);
        obj.maskExist = 1;
    end
    obj.modelType = 255;
    waitbar(1, wb);
end
delete(wb);
drawnow;    % otherwise im_browser crashes after model convertion when leaving the preferences dialog
end