function varargout = mibImageAdjustmentGUI(varargin)
% function varargout = mibImageAdjustmentGUI(varargin)
% mibImageAdjustmentGUI function allows to adjust contrast and gamma of the shown dataset.
%
% mibImageAdjustmentGUI contains MATLAB code for mibImageAdjustmentGUI.fig

% Copyright (C) 23.06.2014, Ilya Belevich, University of Helsinki (ilya.belevich @ helsinki.fi)
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% Updates
% 29.09.2015 - updated the way how the window is reinitialized 
% 24.01.2016 - updated for 4D

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @mibImageAdjustmentGUI_OpeningFcn, ...
    'gui_OutputFcn',  @mibImageAdjustmentGUI_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT
end

% --- Executes just before mibImageAdjustmentGUI is made visible.
function mibImageAdjustmentGUI_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to mibImageAdjustmentGUI (see VARARGIN)

% obtain controller
handles.winController = varargin{1};

repositionSwitch = 1; % reposition the figure, when creating a new figure

% Choose default command line output for mibImageAdjustmentGUI
handles.output = hObject;

% update font and size
global Font;
if handles.colorChannelText.FontSize ~= Font.FontSize ...
        || ~strcmp(handles.colorChannelText.FontName, Font.FontName)
    mibUpdateFontSize(handles.mibImageAdjustmentGUI, Font);
end

% resize all elements x1.25 times for macOS
mibRescaleWidgets(handles.mibImageAdjustmentGUI);
% 
% Adding listeners if the window is opened for the first time
handles.minSliderListener = addlistener(handles.minSlider, 'ContinuousValueChange', @minSlider_Callback);
handles.maxSliderListener = addlistener(handles.maxSlider, 'ContinuousValueChange', @maxSlider_Callback);
handles.gammaSliderListener = addlistener(handles.gammaSlider, 'ContinuousValueChange', @gammaSlider_Callback);

% Update handles structure
guidata(hObject, handles);

% Determine the position of the dialog - on a side of the main figure
% if available, else, centered on the main figure
if repositionSwitch
    FigPos=get(0,'DefaultFigurePosition');
    OldUnits = get(hObject, 'Units');
    set(hObject, 'Units', 'pixels');
    OldPos = get(hObject,'Position');
    FigWidth = OldPos(3);
    FigHeight = OldPos(4);
    if isempty(gcbf)
        ScreenUnits=get(0,'Units');
        set(0,'Units','pixels');
        ScreenSize=get(0,'ScreenSize');
        set(0,'Units',ScreenUnits);
        
        FigPos(1)=1/2*(ScreenSize(3)-FigWidth);
        FigPos(2)=2/3*(ScreenSize(4)-FigHeight);
    else
        GCBFOldUnits = get(gcbf,'Units');
        set(gcbf,'Units','pixels');
        GCBFPos = get(gcbf,'Position');
        set(gcbf,'Units',GCBFOldUnits);
        screenSize = get(0,'ScreenSize');
        if GCBFPos(1)-FigWidth > 0 % put figure on the left side of the main figure
            FigPos(1:2) = [GCBFPos(1)-FigWidth-10 GCBFPos(2)+GCBFPos(4)-FigHeight+59];
        elseif GCBFPos(1) + GCBFPos(3) + FigWidth < screenSize(3) % put figure on the right side of the main figure
            FigPos(1:2) = [GCBFPos(1)+GCBFPos(3)+10 GCBFPos(2)+GCBFPos(4)-FigHeight+59];
        else
            FigPos(1:2) = [(GCBFPos(1) + GCBFPos(3) / 2) - FigWidth / 2, ...
                (GCBFPos(2) + GCBFPos(4) / 2) - FigHeight / 2];
        end
    end
    FigPos(3:4)=[FigWidth FigHeight];
    set(hObject, 'Position', FigPos);
    set(hObject, 'Units', OldUnits);
end
% UIWAIT makes mibImageAdjustmentGUI wait for user response (see UIRESUME)
% uiwait(handles.mibImageAdjustmentGUI);
end

% --- Outputs from this function are returned to the command line.
function varargout = mibImageAdjustmentGUI_OutputFcn(hObject, eventdata, handles)
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;
end

% --- Executes when user attempts to close mibImageAdjustmentGUI.
function mibImageAdjustmentGUI_CloseRequestFcn(hObject, eventdata, handles) %#ok<DEFNU>
handles.winController.closeWindow();
end

% --- Executes on slider movement.
function minSlider_Callback(hObject, eventdata, handles)
handles = guidata(hObject);     % update handles for listener
handles.winController.minSlider_Callback();
end


function minEdit_Callback(hObject, eventdata, handles)
handles.winController.minEdit_Callback();
end

% --- Executes on slider movement.
function maxSlider_Callback(hObject, eventdata, handles)
handles = guidata(hObject);     % update handles for listener
handles.winController.maxSlider_Callback();
end

function maxEdit_Callback(hObject, eventdata, handles)
handles.winController.maxEdit_Callback();
end

% --- Executes on slider movement.
function gammaSlider_Callback(hObject, eventdata, handles)
handles = guidata(hObject);     % update handles for listener
handles.winController.gammaSlider_Callback();
end


function gammaEdit_Callback(hObject, eventdata, handles)
handles.winController.gammaEdit_Callback();
end

% --- Executes on button press in logViewCheck.
function logViewCheck_Callback(hObject, eventdata, handles)
handles.winController.updateHist();
end

% --- Executes on mouse press over figure background, over a disabled or
% --- inactive control, or over an axes background.
function mibImageAdjustmentGUI_WindowButtonDownFcn(hObject, eventdata, handles)
handles.winController.mibImageAdjustmentGUI_WindowButtonDownFcn();
end

function colorChannelCombo_Callback(hObject, eventdata, handles)
% --- Executes on selection change in colorChannelCombo.
handles.winController.colorChannelCombo_Callback();
end

function adjHelpBtn_Callback(hObject, eventdata, handles)
% --- Executes on button press in adjHelpBtn.
handles.winController.adjHelpBtn_Callback();
end


function findMinBtn_Callback(hObject, eventdata, handles)
% --- Executes on button press in findMinBtn.
handles.winController.findMinBtn_Callback();
end

function findMaxBtn_Callback(hObject, eventdata, handles)
% --- Executes on button press in findMaxBtn.
handles.winController.findMaxBtn_Callback();
end

% --- Executes on button press in updateBtn.
function updateBtn_Callback(hObject, eventdata, handles)
handles.winController.updateWidgets();
end

% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over minSlider.
function minSlider_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to minSlider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.winController.minSlider_ButtonDownFcn();
end

% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over maxSlider.
function maxSlider_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to maxSlider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.winController.maxSlider_ButtonDownFcn();
end


% --- If Enable == 'on', executes on mouse press in 5 pixel border.
% --- Otherwise, executes on mouse press in 5 pixel border or over gammaSlider.
function gammaSlider_ButtonDownFcn(hObject, eventdata, handles)
% hObject    handle to gammaSlider (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
handles.gammaSlider.Value = 1;
handles.winController.gammaSlider_Callback();
end

function applyBtn_Callback(hObject, eventdata, handles)
% --- Executes on button press in applyBtn.
handles.winController.applyBtn_Callback();
end

% --- Executes on button press in stretchCurrent.
function stretchCurrent_Callback(hObject, eventdata, handles)
handles.winController.stretchCurrent_Callback();
end


% --- Executes on button press in autoHistCheck.
function autoHistCheck_Callback(hObject, eventdata, handles)
handles.winController.autoHistCheck_Callback();
end
